/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <system_settings.h>
#include <Elementary.h>
#include <efl_extension.h>
#include <media_content.h>
#include <player.h>
#include <app.h>
#include <dlog.h>
#include "$(appName).h"
#include "view.h"
#include "data.h"

#define MEDIA_DIRECTORY "/opt/usr/media"
#define IMG_PATH_NO_ALBUM "images/music_no_album_art.png"

static struct main_info {
	Ecore_Timer *timer_progressbar;
	player_h player;
} s_info = {
	.timer_progressbar = NULL,
	.player = NULL,
};

static void _btn_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static void _btn_up_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static Eina_Bool _progressbar_timer_cb(void *user_data);
static void _start_progressbar(void *user_data);
static void _stop_progressbar(void);
static void _play_current_music(const char *file_path, void *user_data);
static void _change_play_button_image(void *user_data);
static void _play_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _set_no_content_layout(void *user_data, const char *text);
static void _set_current_album(void *user_data, int index);
static void _next_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _prev_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _more_btn_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _player_completed_cb(void *user_data);
static void _content_back_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _scan_folder_cb(media_content_error_e error, void *user_data);

/*
 * @brief: Hook to take necessary actions before main event loop starts
 * Initialize UI resources and application's data
 * If this function returns true, the main loop of application starts
 * If this function returns false, the application is terminated
 */
static bool app_create(void *user_data)
{
	Evas_Object *conform = NULL;
	Evas_Object *content = NULL;
	char *icon_path = NULL;
	char full_path[PATH_MAX] = { 0, };
	char default_img_path[PATH_MAX] = { 0, };

	/*
	 * Connects to the media content service
	 */
	if (media_content_connect() != MEDIA_CONTENT_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Can not connect media content.");
		return false;
	}

	/*
	 * Creates a player handle for playing multimedia content
	 */
	if (player_create(&s_info.player) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Can not create player handler.");
		media_content_disconnect();
		return false;
	}

	/*
	 * Create basic GUI
	 */
	view_create();

	/*
	 * Get EDJ file path will be used for GUI
	 */
	data_get_resource_path(EDJ_FILE, full_path, sizeof(full_path));

	/*
	 * Create GUI for music player
	 */
	conform = view_get_conformant();
	content = view_create_layout_for_conformant(conform, full_path, GRP_MAIN, _content_back_cb, NULL);
	if (content == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a content.");
		return NULL;
	}

	/*
	 * Set default album art
	 */
	data_get_resource_path(IMG_PATH_NO_ALBUM, default_img_path, sizeof(default_img_path));
	view_music_set_album_art(content, "sw.bg", "NULL", default_img_path);

	/*
	 * Set progressbar background image and color
	 */
	icon_path = data_get_image("sw.progressbar.bg");
	view_set_image(content, "sw.progressbar.bg", icon_path);
	view_set_color(content, "sw.progressbar.bg", 0, 87, 107, 255);
	free(icon_path);

	/*
	 * Set Play button and color
	 */
	icon_path = data_get_image("sw.icon.play");
	view_set_button(content, "sw.icon.play", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _play_btn_clicked_cb, content);
	view_set_color(content, "sw.icon.play", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set Prev button and color
	 */
	icon_path = data_get_image("sw.icon.prev");
	view_set_button(content, "sw.icon.prev", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _prev_btn_clicked_cb, content);
	view_set_color(content, "sw.icon.prev", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set Next button and color
	 */
	icon_path = data_get_image("sw.icon.next");
	view_set_button(content, "sw.icon.next", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _next_btn_clicked_cb, content);
	view_set_color(content, "sw.icon.next", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set progressbar that is used to display play time
	 */
	view_set_progressbar(content, "sw.progressbar", 53, 5);
	view_set_color_of_circle_object(content, "sw.progressbar", 0, 192, 235, 255 * 0.5);
	view_set_progressbar_val(content, "sw.progressbar", 0);

	/*
	 * Set more button & more button item
	 */
	view_set_more_button(content, "sw.more", NULL, NULL, NULL, NULL);
	data_get_resource_path(data_get_more_button_image(0), full_path, sizeof(full_path));
	view_add_more_button_item(content,
		"sw.more",
		data_get_more_button_main_text(0),
		data_get_more_button_sub_text(0),
		full_path,
		_more_btn_item_clicked_cb,
		content);

	/*
	 * Requests to scan a media folder, asynchronously
	 */
	if (media_content_scan_folder(MEDIA_DIRECTORY, EINA_TRUE, _scan_folder_cb, content) != MEDIA_CONTENT_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get media content");
	}

	/*
	 * Registers a callback function to be invoked when the playback is finished
	 */
	if (player_set_completed_cb(s_info.player, _player_completed_cb, content) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to register player state changed cb");
	}

	return true;
}

/*
 * @brief: This callback function is called when another application
 * sends the launch request to the application
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/*
 * @brief: This callback function is called each time
 * the application is completely obscured by another application
 * and becomes invisible to the user
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/*
 * @brief: This callback function is called each time
 * the application becomes visible to the user
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/*
 * @brief: This callback function is called once after the main loop of the application exits
 */
static void app_terminate(void *user_data)
{
	/* Release all resources. */


	/*
	 * Unregisters the callback function
	 */
	if (player_unset_completed_cb(s_info.player)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to unset completed cb");
	}

	/*
	 * Destroys the media player handle and releases all its resources
	 */
	if (player_destroy(s_info.player) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Can not destroy player.");
	}

	data_destroy_album_list();
	view_destroy();

	/*
	 * Disconnects from the media content service
	 */
	if (media_content_disconnect() != MEDIA_CONTENT_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Can not disconnect media content.");
	}
}

/*
 * @brief: This function will be called when the language is changed
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;

	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

	if (locale != NULL) {
		elm_language_set(locale);
		free(locale);
	}
	return;
}

/*
 * @brief: Main function of the application
 */
int main(int argc, char *argv[])
{
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	/*
	 * If you want to handling more events,
	 * Please check the application lifecycle guide
	 */
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, NULL);

	ret = ui_app_main(argc, argv, &event_callback, NULL);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);
	}

	return ret;
}

/*
 * @brief: The function operated on button down
 * @param[user_data]: Data to pass to function when it is called
 * @param[e]: An opaque handle to an Evas canvas
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _btn_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info)
{
	evas_object_color_set(obj, 250, 250, 250, 102);
}

/*
 * @brief: The function operated on button up
 * @param[user_data]: Data to pass to function when it is called
 * @param[e]: An opaque handle to an Evas canvas
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _btn_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	evas_object_color_set(obj, 250, 250, 250, 255);

}

/*
 * @brief: Function to update progressbar value
 * @param[user_data]: Data to pass to function when it is called
 */
static Eina_Bool _progressbar_timer_cb(void *user_data)
{
	Evas_Object *content = user_data;
	double ratio = 0;
	int dur_msec = 0, pos_msec = 0;

	player_get_duration(s_info.player, &dur_msec);
	player_get_play_position(s_info.player, &pos_msec);
	ratio = ((double) pos_msec / dur_msec) * 100;
	if (ratio == 100) {
		s_info.timer_progressbar = NULL;
		return ECORE_CALLBACK_CANCEL;
	}

	view_set_progressbar_val(content, "sw.progressbar", ratio);

	return ECORE_CALLBACK_RENEW;
}

/*
 * @brief: Create a timer to update progressbar
 * @param[user_data]: Data to pass to function when it is called
 */
static void _start_progressbar(void *user_data)
{
	Evas_Object *content = user_data;

	if (s_info.timer_progressbar) {
		ecore_timer_del(s_info.timer_progressbar);
		s_info.timer_progressbar = NULL;
	}
	s_info.timer_progressbar = ecore_timer_add(1.0f, _progressbar_timer_cb, content);
}

/*
 * @brief: Remove a timer
 */
static void _stop_progressbar(void)
{
	if (s_info.timer_progressbar) {
		ecore_timer_del(s_info.timer_progressbar);
		s_info.timer_progressbar = NULL;
	}
}

/*
 * @brief: Play a music that is displayed at screen
 * @param[file_path]: File path of music
 * @param[user_data]: Data to pass to function when it is called
 */
static void _play_current_music(const char *file_path, void *user_data)
{
	Evas_Object *content = user_data;
	player_state_e player_state;

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	if (player_state != PLAYER_STATE_IDLE) {
		if (player_unprepare(s_info.player) != PLAYER_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "unprepare error");
			return;
		}
	}

	if (player_set_uri(s_info.player, file_path) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "set uri error");
		return;
	}

	if (player_prepare(s_info.player) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "prepare error");
		return;
	}

	if (player_start(s_info.player) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "start error");
		return;
	}

	_start_progressbar(content);
}

/*
 * @brief: Change a play button icon by player state
 * @param[user_data]: Data to pass to function when it is called
 */
static void _change_play_button_image(void *user_data)
{
	Evas_Object *content = user_data;
	Evas_Object *btn = NULL;
	char *icon_path = NULL;
	char full_path[PATH_MAX] = { 0, };
	player_state_e player_state;

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	switch (player_state) {
	case PLAYER_STATE_NONE:
	case PLAYER_STATE_IDLE:
	case PLAYER_STATE_READY:
	case PLAYER_STATE_PAUSED:
		icon_path = "images/Controller/music_controller_btn_play.png";
		break;
	case PLAYER_STATE_PLAYING:
		icon_path = "images/Controller/music_controller_btn_pause.png";
		break;
	default:
		break;
	}

	data_get_resource_path(icon_path, full_path, sizeof(full_path));

	btn = elm_object_part_content_get(content, "sw.icon.play");
	if (btn) {
		view_set_image(btn, "icon", full_path);
	}
}

/*
 * @brief: The function operated on play button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _play_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content = user_data;
	Eina_List *album_list = NULL;
	int index = 0;
	char *file_path = NULL;
	player_state_e player_state;
	device_info_e device;

	device = (device_info_e) evas_object_data_get(content, "__DEVICE_INFO__");
	if (device == DEVICE_INFO_PHONE) {
		return;
	}

	album_list = data_get_album_list();
	if (album_list == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "album list is NULL");
		_set_no_content_layout(content, "No tracks");
		return;
	}

	index = (int) evas_object_data_get(content, "__CURRENT_ALBUM_INDEX__");
	file_path = data_get_album_file_path(album_list, index);
	if (ecore_file_exists(file_path) != EINA_TRUE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "file is not exist : %s", file_path);
		return;
	}

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	switch (player_state) {
	case PLAYER_STATE_IDLE:
		_play_current_music(file_path, content);
		break;
	case PLAYER_STATE_PLAYING:
		if (player_pause(s_info.player) != PLAYER_ERROR_NONE) {
			return;
		}
		_stop_progressbar();
		break;
	case PLAYER_STATE_PAUSED:
	case PLAYER_STATE_READY:
		if (player_start(s_info.player) != PLAYER_ERROR_NONE) {
			return;
		}
		_start_progressbar(content);
		break;
	case PLAYER_STATE_NONE:
	default:
		break;
	}

	_change_play_button_image(content);
}

/*
 * @brief: Set default album image
 * @param[user_data]: Data to pass to function when it is called
 * @param[text]: Text you want to set to the part
 */
static void _set_no_content_layout(void *user_data, const char *text)
{
	Evas_Object *content = user_data;
	char *icon = NULL;
	char default_img_path[PATH_MAX] = { 0, };
	char edj_path[PATH_MAX] = { 0, };
	device_info_e device;

	data_get_resource_path(IMG_PATH_NO_ALBUM, default_img_path, sizeof(default_img_path));
	data_get_resource_path(EDJ_FILE, edj_path, sizeof(edj_path));

	view_music_set_album_art(content, "sw.bg", "NULL", default_img_path);

	device = (device_info_e) evas_object_data_get(content, "__DEVICE_INFO__");
	icon = data_get_device_image(device);
	view_music_set_title_layout(content, "sw.title.layout", edj_path, icon, text);
	free(icon);

	view_set_text(content, "txt.artist", "");

	_stop_progressbar();
	view_set_progressbar_val(content, "sw.progressbar", 0);

	_change_play_button_image(content);
}

/*
 * @brief: Set current album information(album art, title, artist, device icon)
 * @param[user_data]: The user data to be passed to this functions
 * @param[index]: The order of album list(0 is the first item in album list)
 */
static void _set_current_album(void *user_data, int index)
{
	Evas_Object *content = user_data;
	Eina_List *album_list = NULL;
	char *album_art = NULL;
	char *title = NULL;
	char *artist = NULL;
	char *icon = NULL;
	char default_img_path[PATH_MAX] = { 0, };
	char edj_path[PATH_MAX] = { 0, };
	device_info_e device;

	album_list = data_get_album_list();
	if (album_list == NULL) {
		_set_no_content_layout(content, "No tracks");
		return;
	}

	data_get_resource_path(IMG_PATH_NO_ALBUM, default_img_path, sizeof(default_img_path));
	data_get_resource_path(EDJ_FILE, edj_path, sizeof(edj_path));

	album_art = data_get_album_art(album_list, index);
	view_music_set_album_art(content, "sw.bg", album_art, default_img_path);

	device = (device_info_e) evas_object_data_get(content, "__DEVICE_INFO__");
	icon = data_get_device_image(device);
	title = data_get_album_title(album_list, index);
	view_music_set_title_layout(content, "sw.title.layout", edj_path, icon, title);
	free(icon);

	artist = data_get_album_artist(album_list, index);
	view_set_text(content, "txt.artist", artist);

	_stop_progressbar();
	view_set_progressbar_val(content, "sw.progressbar", 0);
}

/*
 * @brief: The function operated on next button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _next_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content = user_data;
	Eina_List *album_list = NULL;
	char *file_path = NULL;
	int index = 0;
	int count = 0;
	player_state_e player_state;
	device_info_e device;

	device = (device_info_e) evas_object_data_get(content, "__DEVICE_INFO__");
	if (device == DEVICE_INFO_PHONE) {
		return;
	}

	album_list = data_get_album_list();
	if (album_list == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "album list is NULL");
		_set_no_content_layout(content, "No tracks");
		return;
	}

	index = (int) evas_object_data_get(content, "__CURRENT_ALBUM_INDEX__");
	count = eina_list_count(album_list);
	index = (index + 1) % count;

	file_path = data_get_album_file_path(album_list, index);
	if (ecore_file_exists(file_path) != EINA_TRUE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "file is not exist : %s", file_path);
		album_list = data_remove_album_data_by_file_path(file_path);
		count = eina_list_count(album_list);
		if (count == 0) {
			_set_no_content_layout(content, "No tracks");
			return;
		}
		index = (index + 1) % count;
	}

	evas_object_data_set(content, "__CURRENT_ALBUM_INDEX__", (void *) index);
	_set_current_album(content, index);

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	if (player_state == PLAYER_STATE_PLAYING) {
		file_path = data_get_album_file_path(album_list, index);
		_play_current_music(file_path, content);
	} else {
		if (player_unprepare(s_info.player) != PLAYER_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "unprepare error");
			return;
		}
	}

	_change_play_button_image(content);
}

/*
 * @brief: The function operated on prev button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _prev_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content = user_data;
	Eina_List *album_list = NULL;
	char *file_path = NULL;
	int index = 0;
	int count = 0;
	player_state_e player_state;
	device_info_e device;

	device = (device_info_e) evas_object_data_get(content, "__DEVICE_INFO__");
	if (device == DEVICE_INFO_PHONE) {
		return;
	}

	album_list = data_get_album_list();
	if (album_list == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "album list is NULL");
		_set_no_content_layout(content, "No tracks");
		return;
	}

	index = (int) evas_object_data_get(content, "__CURRENT_ALBUM_INDEX__");
	count = eina_list_count(album_list);
	index = (index == 0) ? count - 1 : index - 1;

	file_path = data_get_album_file_path(album_list, index);
	if (ecore_file_exists(file_path) != EINA_TRUE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "file is not exist : %s", file_path);
		album_list = data_remove_album_data_by_file_path(file_path);
		count = eina_list_count(album_list);
		if (count == 0) {
			_set_no_content_layout(content, "No tracks");
			return;
		}
		index = (index == 0) ? count - 1 : index - 1;
	}

	evas_object_data_set(content, "__CURRENT_ALBUM_INDEX__", (void *) index);
	_set_current_album(content, index);

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	if (player_state == PLAYER_STATE_PLAYING) {
		file_path = data_get_album_file_path(album_list, index);
		_play_current_music(file_path, content);
	} else {
		if (player_unprepare(s_info.player) != PLAYER_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "unprepare error");
			return;
		}
	}

	_change_play_button_image(content);
}

/*
 * @brief: The function operated on more button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _more_btn_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content = user_data;
	Eext_Object_Item *item = (Eext_Object_Item *) event_info;
	const char *sub_text;
	char full_path[PATH_MAX] = { 0, };
	device_info_e device;

	sub_text = eext_more_option_item_part_text_get(item, "selector,sub_text");

	if (!strcmp(sub_text, data_get_more_button_sub_text(DEVICE_INFO_GEAR))) {
		device = DEVICE_INFO_PHONE;
		evas_object_data_set(content, "__DEVICE_INFO__", (void *) device);
		_set_no_content_layout(content, "BT disconnected");
	} else {
		device = DEVICE_INFO_GEAR;
		evas_object_data_set(content, "__DEVICE_INFO__", (void *) device);
		evas_object_data_set(content, "__CURRENT_ALBUM_INDEX__", (void *) 0);
		_set_current_album(content, 0);
	}

	data_get_resource_path(data_get_more_button_image(device), full_path, sizeof(full_path));
	view_music_change_more_item(item, data_get_more_button_main_text(device), data_get_more_button_sub_text(device), full_path);

	if (player_unprepare(s_info.player) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "unprepare error");
		return;
	}

	_change_play_button_image(content);
}

/*
 * @brief: Called when the media player is completed
 * @param[user_data]: Data passed from the callback registration function
 */
static void _player_completed_cb(void *user_data)
{
	Evas_Object *content = user_data;

	_next_btn_clicked_cb(content, NULL, NULL);

	return;
}

/*
 * @brief: The function operated on back key event
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _content_back_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	player_state_e player_state;

	if (player_get_state(s_info.player, &player_state) != PLAYER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get player state");
		return;
	}

	if (player_state == PLAYER_STATE_PLAYING) {
		dlog_print(DLOG_DEBUG, LOG_TAG, "lowers the window object");
		Evas_Object *win = NULL;
		win = view_get_window();
		elm_win_lower(win);
	} else {
		dlog_print(DLOG_DEBUG, LOG_TAG, "exit application");
		ui_app_exit();
	}
}

/*
 * @brief: Called when the media scanning is finished
 * @param[error]: Enumeration for a media content error
 * @param[user_data]: Data needed in this function
 */
static void _scan_folder_cb(media_content_error_e error, void *user_data)
{
	Evas_Object *content = user_data;
	Eina_List *album_list = NULL;

	if (error != MEDIA_CONTENT_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to scan media folder(%d)", error);
		return;
	}

	/*
	 * Create a album list from the media database
	 */
	album_list = data_create_album_list();

	/*
	 * Set first item in album list
	 */
	evas_object_data_set(content, "__DEVICE_INFO__", (void *) DEVICE_INFO_GEAR);
	evas_object_data_set(content, "__CURRENT_ALBUM_INDEX__", (void *) 0);
	_set_current_album(content, 0);
}
